@@ -55,6 +55,7 @@
 #include "print.h"
 #include <e-util/e-icon-factory.h>
 #include "e-cal-popup.h"
+#include "misc.h"
 
 extern ECompEditorRegistry *comp_editor_registry;
 
@@ -89,6 +90,9 @@ static struct tm e_calendar_table_get_current_time (ECellDateEdit *ecde,
 						    gpointer data);
 static void mark_row_complete_cb (int model_row, gpointer data);
 
+static void hide_completed_rows (ECalModel *model, GList *clients_list, char *hide_sexp, GPtrArray *comp_objects);
+static void show_completed_rows (ECalModel *model, GList *clients_list, char *show_sexp, GPtrArray *comp_objects);
+
 /* Signal IDs */
 enum {
 	USER_CREATED,
@@ -938,6 +942,7 @@ e_calendar_table_open_task (ECalendarTable *cal_table, ECalModelComponent *comp_
 {
 	CompEditor *tedit;
 	const char *uid;
+	guint32 flags = 0;
 	
 	uid = icalcomponent_get_uid (comp_data->icalcomp);
 
@@ -945,17 +950,25 @@ e_calendar_table_open_task (ECalendarTable *cal_table, ECalModelComponent *comp_
 	if (tedit == NULL) {
 		ECalComponent *comp;
 
-		tedit = COMP_EDITOR (task_editor_new (comp_data->client, assign));
-
 		comp = e_cal_component_new ();
 		e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (comp_data->icalcomp));
+
+		if (assign) {
+			flags |= COMP_EDITOR_IS_ASSIGNED;
+
+			if (itip_organizer_is_user (comp, comp_data->client) || 
+					!e_cal_component_has_attendees (comp))
+				flags |= COMP_EDITOR_USER_ORG;
+		}
+
+		tedit = COMP_EDITOR (task_editor_new (comp_data->client, flags));
 		comp_editor_edit_comp (tedit, comp);
-		if (assign)
+	
+		if (flags & COMP_EDITOR_IS_ASSIGNED)
 			task_editor_show_assignment (TASK_EDITOR (tedit));
 		
 		e_comp_editor_registry_add (comp_editor_registry, tedit, FALSE);
 	}
-	
 	comp_editor_focus (tedit);
 }
 
@@ -1166,7 +1179,7 @@ static EPopupItem tasks_popup_items [] = {
 
 	{ E_POPUP_ITEM, "80.assign", N_("_Assign Task"), e_calendar_table_on_assign, NULL, NULL, E_CAL_POPUP_SELECT_ONE, E_CAL_POPUP_SELECT_EDITABLE|E_CAL_POPUP_SELECT_ASSIGNABLE },
 	{ E_POPUP_ITEM, "90.forward", N_("_Forward as iCalendar"), e_calendar_table_on_forward, NULL, "stock_mail-forward", E_CAL_POPUP_SELECT_ONE },
-	{ E_POPUP_ITEM, "a0.markonecomplete", N_("_Mark as Complete"), mark_as_complete_cb, NULL, NULL, E_CAL_POPUP_SELECT_ONE, E_CAL_POPUP_SELECT_EDITABLE },
+	{ E_POPUP_ITEM, "a0.markonecomplete", N_("_Mark as Complete"), mark_as_complete_cb, NULL, NULL, E_CAL_POPUP_SELECT_ONE, E_CAL_POPUP_SELECT_EDITABLE | E_CAL_POPUP_SELECT_NOTCOMPLETE},
 	{ E_POPUP_ITEM, "b0.markmanycomplete", N_("_Mark Selected Tasks as Complete"), mark_as_complete_cb, NULL, NULL, E_CAL_POPUP_SELECT_MANY, E_CAL_POPUP_SELECT_EDITABLE },
 
 	{ E_POPUP_BAR, "c0.bar" },
@@ -1265,6 +1278,74 @@ e_calendar_table_on_key_press (ETable *table,
 	return FALSE;
 }
 
+static void
+hide_completed_rows (ECalModel *model, GList *clients_list, char *hide_sexp, GPtrArray *comp_objects)
+{
+	GList *l, *m, *objects;
+	ECal *client;
+	int pos;
+
+	for (l = clients_list; l != NULL; l = l->next) {
+		client = l->data;
+
+		if (!e_cal_get_object_list (client, hide_sexp, &objects, NULL)) {
+			g_warning (G_STRLOC ": Could not get the objects");
+
+			continue;
+		}
+
+		for (m = objects; m; m = m->next) {
+			ECalModelComponent *comp_data;
+
+			if ((comp_data =  e_cal_model_get_component_for_uid (model, icalcomponent_get_uid (m->data)))) {
+				e_table_model_pre_change (E_TABLE_MODEL (model));
+				pos = get_position_in_array (comp_objects, comp_data);
+				e_table_model_row_deleted (E_TABLE_MODEL (model), pos);
+
+				g_ptr_array_remove (comp_objects, comp_data);
+			}
+		}
+
+		g_list_foreach (objects, (GFunc) icalcomponent_free, NULL);
+		g_list_free (objects);
+	}
+}
+
+static void
+show_completed_rows (ECalModel *model, GList *clients_list, char *show_sexp, GPtrArray *comp_objects)
+{
+	GList *l, *m, *objects;
+	ECal *client;
+
+	for (l = clients_list; l != NULL; l = l->next) {
+		client = l->data;
+
+		if (!e_cal_get_object_list (client, show_sexp, &objects, NULL)) {
+			g_warning (G_STRLOC ": Could not get the objects");
+
+			continue;
+		}
+
+		for (m = objects; m; m = m->next) {
+			ECalModelComponent *comp_data;
+
+			if (!(e_cal_model_get_component_for_uid (model, icalcomponent_get_uid (m->data)))) {
+				e_table_model_pre_change (E_TABLE_MODEL (model));
+				comp_data = g_new0 (ECalModelComponent, 1);
+				comp_data->client = client;
+				comp_data->icalcomp = icalcomponent_new_clone (m->data);
+				e_cal_model_set_instance_times (comp_data,
+						e_cal_model_get_timezone (model));
+				comp_data->dtstart = comp_data->dtend = comp_data->due = comp_data->completed = NULL;
+				comp_data->color = NULL;
+
+				g_ptr_array_add (comp_objects, comp_data);
+				e_table_model_row_inserted (E_TABLE_MODEL (model), comp_objects->len - 1);
+			} 
+		}
+	}
+}
+
 /* Loads the state of the table (headers shown etc.) from the given file. */
 void
 e_calendar_table_load_state	(ECalendarTable *cal_table,
@@ -1376,3 +1457,48 @@ e_calendar_table_set_status_message (ECalendarTable *cal_table, const gchar *mes
                 e_activity_handler_operation_progressing (cal_table->activity_handler, cal_table->activity_id, message, progress);
 	}
 }
+
+/**
+ * e_calendar_table_hide_completed_tasks:
+ * @table: A calendar table model.
+ * @client_list: Clients List
+ *
+ * Hide completed tasks.
+ */
+void
+e_calendar_table_process_completed_tasks (ECalendarTable *table, GList *clients_list, gboolean config_changed)
+{
+	ECalModel *model;
+	static GMutex *mutex = NULL;
+	char *hide_sexp, *show_sexp;
+	GPtrArray *comp_objects = NULL;
+
+	if (!mutex)
+		mutex = g_mutex_new ();
+
+	g_mutex_lock (mutex);
+
+	model = e_calendar_table_get_model (table); 
+	comp_objects = e_cal_model_get_object_array (model);
+
+	hide_sexp = calendar_config_get_hide_completed_tasks_sexp (TRUE);
+	show_sexp = calendar_config_get_hide_completed_tasks_sexp (FALSE);
+
+	/* If hide option is unchecked */
+	if (!(hide_sexp && show_sexp))
+		show_sexp = g_strdup ("(is-completed?)");
+
+	/* Delete rows from model*/
+	if (hide_sexp) {
+		hide_completed_rows (model, clients_list, hide_sexp, comp_objects);
+	}
+
+	/* Insert rows into model */
+	if (config_changed) {
+		show_completed_rows (model, clients_list, show_sexp, comp_objects);
+	}
+
+	g_free (hide_sexp);
+	g_free (show_sexp);
+	g_mutex_unlock (mutex);
+}
