@@ -39,6 +39,7 @@
 #include "e-util/e-time-utils.h"
 #include "shell/e-user-creatable-items-handler.h"
 #include <libedataserver/e-url.h>
+#include <libedataserver/e-categories.h>
 #include <libecal/e-cal-time-util.h>
 #include "widgets/menus/gal-view-menus.h"
 #include "dialogs/delete-error.h"
@@ -55,6 +56,7 @@
 #include "e-tasks.h"
 #include "common/authentication.h"
 #include "e-cal-menu.h"
+#include "e-cal-model-tasks.h"
 
 
 /* Private part of the GnomeCalendar structure */
@@ -101,7 +103,6 @@ static void update_view (ETasks *tasks);
 
 static void config_categories_changed_cb (EConfigListener *config_listener, const char *key, gpointer user_data);
 static void backend_error_cb (ECal *client, const char *message, gpointer data);
-ECalMenu *gnome_tasks_get_tasks_menu (ETasks *tasks);
 
 /* Signal IDs */
 enum {
@@ -167,7 +168,7 @@ table_cursor_change_cb (ETable *etable, int row, gpointer data)
 }
 
 ECalMenu *
-gnome_tasks_get_tasks_menu (ETasks *tasks)
+e_tasks_get_tasks_menu (ETasks *tasks)
 {
         g_return_val_if_fail (E_IS_TASKS (tasks), NULL);
 
@@ -291,7 +292,7 @@ update_view (ETasks *tasks)
 
 	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
 		
-	if ((new_sexp = calendar_config_get_hide_completed_tasks_sexp()) != NULL) {
+	if ((new_sexp = calendar_config_get_hide_completed_tasks_sexp (FALSE)) != NULL) {
 		real_sexp = g_strdup_printf ("(and %s %s)", new_sexp, priv->sexp);
 		e_cal_model_set_search_query (model, real_sexp);
 		g_free (new_sexp);
@@ -302,10 +303,31 @@ update_view (ETasks *tasks)
 	e_cal_component_preview_clear (E_CAL_COMPONENT_PREVIEW (priv->preview));
 }
 
+static void
+process_completed_tasks (ETasks *tasks, gboolean config_changed)
+{
+	ETasksPrivate *priv;
+	ECalModel *model;
+
+	g_return_if_fail (tasks != NULL);
+	g_return_if_fail (E_IS_TASKS (tasks));
+
+	priv = tasks->priv;
+
+	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
+
+	e_calendar_table_process_completed_tasks (e_tasks_get_calendar_table (tasks), priv->clients_list, config_changed);
+}
+
 static gboolean
 update_view_cb (ETasks *tasks)
 {	
-	update_view (tasks);
+	ECalModel *model;
+
+	model = e_calendar_table_get_model (E_CALENDAR_TABLE (tasks->priv->tasks_view));
+
+	process_completed_tasks (tasks, FALSE);
+	e_cal_model_tasks_update_due_tasks (E_CAL_MODEL_TASKS (model));
 
 	return TRUE;
 }
@@ -313,7 +335,7 @@ update_view_cb (ETasks *tasks)
 static void
 config_hide_completed_tasks_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 {
-	update_view (data);
+	process_completed_tasks (data, TRUE);
 }
 
 static void
@@ -461,7 +483,6 @@ table_drag_data_delete (ETable         *table,
 	ECalModelComponent *comp_data;
 	ECalModel *model;
 	gboolean read_only = TRUE;
-	
 	priv = tasks->priv;
 	
 	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
@@ -471,7 +492,8 @@ table_drag_data_delete (ETable         *table,
 	if (read_only)
 		return;
 
-	e_cal_remove_object (comp_data->client, icalcomponent_get_uid (comp_data->icalcomp), NULL);
+	if(gdk_drag_drop_succeeded(context))
+		e_cal_remove_object (comp_data->client, icalcomponent_get_uid (comp_data->icalcomp), NULL);
 }
 
 #define E_TASKS_TABLE_DEFAULT_STATE					\
@@ -641,7 +663,7 @@ config_categories_changed_cb (EConfigListener *config_listener, const char *key,
 		cat_list = g_list_remove (cat_list, cat_list->data);
 	}
 
-	cal_search_bar_set_categories (priv->search_bar, cat_array);
+	cal_search_bar_set_categories ((CalSearchBar *)priv->search_bar, cat_array);
 
 	g_ptr_array_free (cat_array, TRUE);
 }
@@ -952,6 +974,7 @@ e_tasks_new_task			(ETasks		*tasks)
 	ECalComponent *comp;
 	const char *category;
 	ECal *ecal;
+	guint32 flags = 0;
 	
 	g_return_if_fail (E_IS_TASKS (tasks));
 
@@ -962,12 +985,14 @@ e_tasks_new_task			(ETasks		*tasks)
 	if (!ecal)
 		return;
 	
+	flags |= COMP_EDITOR_NEW_ITEM | COMP_EDITOR_USER_ORG;
+	
 	comp = cal_comp_task_new_with_defaults (ecal);
 
 	category = cal_search_bar_get_category (CAL_SEARCH_BAR (priv->search_bar));
 	e_cal_component_set_categories (comp, category);
 
-	tedit = task_editor_new (ecal, FALSE);
+	tedit = task_editor_new (ecal, flags);
 	comp_editor_edit_comp (COMP_EDITOR (tedit), comp);
 	g_object_unref (comp);
 
