@@ -384,19 +384,24 @@ emmp_header_add_header (GtkWidget *widget, EMMailerPrefs *prefs)
 {
 	GtkTreeModel *model = GTK_TREE_MODEL (prefs->header_list_store);
 	GtkTreeIter iter;
-	
-	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-	gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
-			    HEADER_LIST_NAME_COLUMN, gtk_entry_get_text (prefs->entry_header), 
-			    HEADER_LIST_ENABLED_COLUMN, TRUE, 
-			    HEADER_LIST_HEADER_COLUMN, gtk_entry_get_text (prefs->entry_header), 
-			    HEADER_LIST_IS_DEFAULT_COLUMN, FALSE, 
-			    -1);
-	gtk_entry_set_text (prefs->entry_header, "");
-	emmp_header_remove_sensitivity (prefs);
-	emmp_header_add_sensitivity (prefs);
-	
-	emmp_save_headers (prefs);
+	const gchar *text = gtk_entry_get_text (prefs->entry_header);
+
+	g_strstrip (text);
+
+	if (text && (strlen (text)>0)) {
+		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
+				HEADER_LIST_NAME_COLUMN, text, 
+				HEADER_LIST_ENABLED_COLUMN, TRUE, 
+				HEADER_LIST_HEADER_COLUMN, text, 
+				HEADER_LIST_IS_DEFAULT_COLUMN, FALSE, 
+				-1);
+		gtk_entry_set_text (prefs->entry_header, "");
+		emmp_header_remove_sensitivity (prefs);
+		emmp_header_add_sensitivity (prefs);
+
+		emmp_save_headers (prefs);
+	}
 }
 
 static void
@@ -814,6 +819,7 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs)
 	font = gconf_client_get_string (prefs->gconf, "/apps/evolution/mail/display/fonts/variable", NULL);
 	prefs->font_variable = GNOME_FONT_PICKER (glade_xml_get_widget (gui, "FontVariable"));
 	gnome_font_picker_set_font_name (prefs->font_variable, font);
+	g_free (font);
 	g_object_set_data ((GObject *) prefs->font_variable, "key", "/apps/evolution/mail/display/fonts/variable");
 	g_signal_connect (prefs->font_variable, "font-set", G_CALLBACK (font_changed), prefs);
 	if (!gconf_client_key_is_writable (prefs->gconf, "/apps/evolution/mail/display/fonts/variable", NULL))
