@@ -226,7 +226,7 @@ em_utils_check_user_can_send_mail (GtkWidget *parent)
 	return TRUE;
 }
 
-/* Editing Filters/vFolders... */
+/* Editing Filters/Search Folders... */
 
 static GtkWidget *filter_editor = NULL;
 
@@ -250,10 +250,10 @@ em_filter_editor_response (GtkWidget *dialog, int button, gpointer user_data)
 	filter_editor = NULL;
 }
 
-static const char *em_filter_source_element_names[] = {
-	"incoming",
-	"outgoing",
-	NULL,
+static EMFilterSource em_filter_source_element_names[] = {
+	{ "incoming", },
+	{ "outgoing", },
+	{ 0 }
 };
 
 /**
@@ -286,7 +286,12 @@ em_utils_edit_filters (GtkWidget *parent)
 		e_error_run((GtkWindow *)parent, "mail:filter-load-error", ((RuleContext *)fc)->error, NULL);
 		return;
 	}
-	
+
+	if (em_filter_source_element_names[0].name == NULL) {
+		em_filter_source_element_names[0].name = _("Incoming");
+		em_filter_source_element_names[1].name = _("Outgoing");
+	}
+
 	filter_editor = (GtkWidget *) em_filter_editor_new (fc, em_filter_source_element_names);
 	if (parent != NULL)
 		e_dialog_set_transient_for ((GtkWindow *) filter_editor, parent);
@@ -356,14 +361,15 @@ emu_get_save_filesel (GtkWidget *parent, const char *title, const char *name, Gt
 }
 
 static void
-emu_update_save_path(const char *filename)
+emu_update_save_path(const char *filename, gboolean path)
 {
-	char *dir = g_path_get_dirname(filename);
+	char *dir = path ? (char *)filename : g_path_get_dirname(filename);
 	GConfClient *gconf = gconf_client_get_default();
 
 	gconf_client_set_string(gconf, "/apps/evolution/mail/save_dir", dir, NULL);
 	g_object_unref(gconf);
-	g_free(dir);
+	if (!path)
+		g_free(dir);
 }
 
 static gboolean
@@ -405,7 +411,7 @@ emu_save_part_response(GtkWidget *filesel, int response, CamelMimePart *part)
 		if (!emu_can_save((GtkWindow *)filesel, path))
 			return;
 
-		emu_update_save_path(path);
+		emu_update_save_path(path, FALSE);
 		/* FIXME: popup error if it fails? */
 		mail_save_part(part, path, NULL, NULL);
 	}
@@ -457,7 +463,7 @@ emu_save_parts_response (GtkWidget *filesel, int response, GSList *parts)
                 path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
 #endif
 
-                emu_update_save_path(path);
+                emu_update_save_path(path, TRUE);
 
                 for ( selected = parts; selected != NULL; selected = selected->next) {
                         const char *file_name;
@@ -575,7 +581,7 @@ emu_save_messages_response(GtkWidget *filesel, int response, struct _save_messag
 		if (!emu_can_save((GtkWindow *)filesel, path))
 			return;
 
-		emu_update_save_path(path);
+		emu_update_save_path(path, FALSE);
 		mail_save_messages(data->folder, data->uids, path, NULL, NULL);
 		data->uids = NULL;
 	}
