@@ -249,6 +249,7 @@ config_write_style (void)
 	gboolean custom;
 	char *fix_font;
 	char *var_font;
+	char *citation_color;
 	FILE *rc;
 	
 	if (!(rc = fopen (config->gtkrc, "wt"))) {
@@ -259,7 +260,7 @@ config_write_style (void)
 	custom = gconf_client_get_bool (config->gconf, "/apps/evolution/mail/display/fonts/use_custom", NULL);
 	var_font = gconf_client_get_string (config->gconf, "/apps/evolution/mail/display/fonts/variable", NULL);
 	fix_font = gconf_client_get_string (config->gconf, "/apps/evolution/mail/display/fonts/monospace", NULL);
-
+	citation_color = gconf_client_get_string (config->gconf, "/apps/evolution/mail/display/citation_colour", NULL);
  	CONFIG_GET_SPELL_VALUE (int, "/spell_error_color_red",   red, (void)0, (int));
  	CONFIG_GET_SPELL_VALUE (int, "/spell_error_color_green", green, (void)0, (int));
  	CONFIG_GET_SPELL_VALUE (int, "/spell_error_color_blue",  blue, (void)0, (int));
@@ -270,7 +271,8 @@ config_write_style (void)
 
 	if (gconf_client_get_bool (config->gconf, "/apps/evolution/mail/display/mark_citations", NULL))
 		fprintf (rc, "        GtkHTML::cite_color = \"%s\"\n",
-			 gconf_client_get_string (config->gconf, "/apps/evolution/mail/display/citation_colour", NULL));
+			 citation_color);
+	g_free (citation_color);
 
 	if (custom && var_font && fix_font) {
 		fprintf (rc,
@@ -278,6 +280,9 @@ config_write_style (void)
 			 "        font_name = \"%s\"\n",
 			 fix_font, var_font); 
 	}
+	g_free (fix_font);
+	g_free (var_font);
+
 	fprintf (rc, "}\n\n");
 	
 	fprintf (rc, "widget \"*.EMFolderView.*.GtkHTML\" style \"evolution-mail-custom-fonts\"\n");
@@ -420,14 +425,14 @@ mail_config_write_on_exit (void)
 		
 		account = (EAccount *) e_iterator_get (iter);
 		
-		if (account->source->save_passwd && account->source->url) {
+		if (account->source->save_passwd && account->source->url && account->source->url[0]) {
 			passwd = mail_session_get_password (account->source->url);
 			mail_session_forget_password (account->source->url);
 			mail_session_add_password (account->source->url, passwd);
 			g_free (passwd);
 		}
 		
-		if (account->transport->save_passwd && account->transport->url) {
+		if (account->transport->save_passwd && account->transport->url && account->transport->url[0]) {
 			passwd = mail_session_get_password (account->transport->url);
 			mail_session_forget_password (account->transport->url);
 			mail_session_add_password (account->transport->url, passwd);
@@ -447,10 +452,10 @@ mail_config_write_on_exit (void)
 	while (e_iterator_is_valid (iter)) {
 		account = (EAccount *) e_iterator_get (iter);
 		
-		if (account->source->save_passwd && account->source->url)
+		if (account->source->save_passwd && account->source->url && account->source->url[0])
 			mail_session_remember_password (account->source->url);
 		
-		if (account->transport->save_passwd && account->transport->url)
+		if (account->transport->save_passwd && account->transport->url && account->transport->url[0])
 			mail_session_remember_password (account->transport->url);
 		
 		e_iterator_next (iter);
@@ -597,7 +602,7 @@ mail_config_get_account_by_source_url (const char *source_url)
 	while (e_iterator_is_valid (iter)) {
 		account = (EAccount *) e_iterator_get (iter);
 		
-		if (account->source && account->source->url) {
+		if (account->source && account->source->url && account->source->url[0]) {
 			CamelURL *url;
 			
 			url = camel_url_new (account->source->url, NULL);
@@ -645,7 +650,7 @@ mail_config_get_account_by_transport_url (const char *transport_url)
 	while (e_iterator_is_valid (iter)) {
 		account = (EAccount *) e_iterator_get (iter);
 		
-		if (account->transport && account->transport->url) {
+		if (account->transport && account->transport->url && account->transport->url[0]) {
 			CamelURL *url;
 			
 			url = camel_url_new (account->transport->url, NULL);
@@ -671,6 +676,24 @@ mail_config_get_account_by_transport_url (const char *transport_url)
 	return NULL;
 }
 
+int
+mail_config_has_proxies (EAccount *account)
+{
+	return e_account_list_account_has_proxies (config->accounts, account);
+}
+
+void
+mail_config_remove_account_proxies (EAccount *account)
+{
+	e_account_list_remove_account_proxies (config->accounts, account);
+}
+
+void
+mail_config_prune_proxies (void)
+{
+	e_account_list_prune_proxies (config->accounts);
+}
+
 EAccountList *
 mail_config_get_accounts (void)
 {
@@ -718,7 +741,7 @@ mail_config_get_default_transport (void)
 	EIterator *iter;
 	
 	account = mail_config_get_default_account ();
-	if (account && account->transport && account->transport->url)
+	if (account && account->transport && account->transport->url && account->transport->url[0])
 		return account->transport;
 	
 	/* return the first account with a transport? */
@@ -726,7 +749,7 @@ mail_config_get_default_transport (void)
 	while (e_iterator_is_valid (iter)) {
 		account = (EAccount *) e_iterator_get (iter);
 		
-		if (account->transport && account->transport->url) {
+		if (account->transport && account->transport->url && account->transport->url[0]) {
 			g_object_unref (iter);
 			
 			return account->transport;
