@@ -73,6 +73,8 @@
 
 static GObjectClass *parent_class = NULL;
 
+static int proxy_page_changed_cb (GtkNotebook *notebook, GtkNotebookPage *page, int num, EAccount *account);
+
 struct _proxyDialogPrivate {
 	/* Glade XML data for the Add/Edit Proxy dialog*/
 	GladeXML *xml;
@@ -104,6 +106,8 @@ struct _proxyDialogPrivate {
 	GtkWidget *options;
 	GtkWidget *private;
         char *help_section;
+
+	GList *proxy_list;
 };
 
 static void
@@ -137,10 +141,11 @@ free_proxy_handler (proxyHandler *handler)
 void
 free_proxy_list (GList *proxy_list)
 {
-	g_list_foreach (proxy_list, (GFunc) free_proxy_handler, NULL);
-	g_list_free (proxy_list);
-	proxy_list = NULL;
-
+	if (proxy_list) {
+		g_list_foreach (proxy_list, (GFunc) free_proxy_handler, NULL);
+		g_list_free (proxy_list);
+		proxy_list = NULL;
+	}
 }
 
 static void
@@ -156,9 +161,10 @@ proxy_dialog_finalize (GObject *object)
 		g_object_unref (priv->proxy_name_selector);
 
 	if (priv) {
+		free_proxy_list (priv->proxy_list);
 		g_free (priv->help_section);
 		g_object_unref (priv->xml_tab);
-		g_object_unref (prd->priv);
+		g_free (prd->priv);
 		prd->priv = NULL;
 	}
 	
@@ -214,6 +220,8 @@ proxy_dialog_init (GObject *object)
 	priv->options = NULL;
 	priv->private = NULL;
 	priv->help_section = NULL;
+
+	priv->proxy_list = NULL;
 }
 
 GType
@@ -336,22 +344,38 @@ proxy_dialog_store_widgets_data (EAccount *account, gint32 dialog)
 					e_error_run (NULL, "org.gnome.evolution.proxy:no-user",NULL ,NULL);
 					return -1; 
 				}
-
+				
 				for (; tmp != NULL; tmp = g_list_next (tmp)) {
 					email = NULL;
 					email = (char *)e_destination_get_email (tmp->data);
 
-					if (g_strrstr (email, "@") == NULL ) {
-						e_error_run (NULL, "org.gnome.evolution.proxy:invalid-user",email ,NULL);
+					if (g_str_equal(email, ""))
+						continue;
+					
+					if (g_strrstr (email, "@") == NULL) {
+						e_error_run (NULL, "org.gnome.evolution.proxy:invalid-user", email, NULL);
 						return -1;
 					} 	
+					if (! g_ascii_strcasecmp(e_gw_connection_get_user_email (prd->cnc), email)) {
+						e_error_run (NULL, "org.gnome.evolution.proxy:invalid-user", email, NULL);
+						return -1;
+					}
 
 					/*check whether already exists*/
-					existing_list = g_object_get_data ( (GObject*)account, "proxy_list");
+					existing_list = priv->proxy_list;
 
 					for (;existing_list; existing_list = g_list_next(existing_list)) {
 						new_proxy = (proxyHandler *) existing_list->data;
 						if ( !g_ascii_strcasecmp (new_proxy->proxy_email, email) ) {
+							if (new_proxy->flags & E_GW_PROXY_DELETED) {
+								new_proxy->permissions = proxy_get_permissions_from_dialog (account);
+								if (new_proxy->flags & E_GW_PROXY_NEW)
+									new_proxy->flags = E_GW_PROXY_NEW;
+								else
+									new_proxy->flags = E_GW_PROXY_EDITED;
+								return 0;
+							}
+
 							
 							e_error_run (NULL, "org.gnome.evolution.proxy:user-is-proxy",email ,NULL);
 							return -1;
@@ -363,6 +387,10 @@ proxy_dialog_store_widgets_data (EAccount *account, gint32 dialog)
 				for (; tmp != NULL; tmp = g_list_next (tmp)) {
 					name = NULL; email = NULL;
 					email = (char *) e_destination_get_email (tmp->data);
+
+					if (g_str_equal(email, ""))
+						continue;
+
 					name = (char *) e_destination_get_name (tmp->data);
 					new_proxy = (proxyHandler *) g_malloc (sizeof (proxyHandler));
 
@@ -375,7 +403,8 @@ proxy_dialog_store_widgets_data (EAccount *account, gint32 dialog)
 					new_proxy->uniqueid = NULL;
 					new_proxy->flags =  E_GW_PROXY_NEW;
 					new_proxy->permissions = proxy_get_permissions_from_dialog (account);
-					g_object_set_data_full ((GObject *) account, "proxy_list", g_list_append (g_object_get_data ((GObject*) account,"proxy_list"), new_proxy), (GDestroyNotify) free_proxy_list);
+				
+					priv->proxy_list = g_list_append (priv->proxy_list, new_proxy);
 				}
 			}
 			break;
@@ -493,44 +522,40 @@ proxy_abort (GtkWidget *button, EConfigHookItemFactoryData *data)
 	EMConfigTargetAccount *target_account;
 	EAccount *account;
 	proxyDialog *prd = NULL;
-	GList *proxy_list;
 
 	target_account = (EMConfigTargetAccount *)data->config->target;
 	account = target_account->account;
 	prd = g_object_get_data ((GObject *)account, "prd");    
-	proxy_list = (GList *) g_object_get_data ((GObject *) account, "proxy_dialog");
 	
-	if (prd == NULL || proxy_list == NULL)
+	if (!prd || !prd->priv || !prd->priv->proxy_list)
 		return;
 
-	g_list_foreach (proxy_list, (GFunc) free_proxy_handler, NULL);
-	g_list_free (proxy_list);
-	proxy_list = NULL;
-
 	g_object_unref (prd);
 	prd = NULL;
 }
+
 void 
 proxy_commit (GtkWidget *button, EConfigHookItemFactoryData *data)
 {
 	EAccount *account;
 	EMConfigTargetAccount *target_account;
 	proxyDialogPrivate *priv;
-	GList *l, *proxy_list;
+	GList *list_iter;
 	proxyHandler *aclInstance;
 	proxyDialog *prd = NULL;
 
 	target_account = (EMConfigTargetAccount *)data->config->target;
 	account = target_account->account;
 	prd = g_object_get_data ((GObject *)account, "prd");    
-	l = g_object_get_data ( (GObject *)account, "proxy_list") ;
 
-	if (prd == NULL || l == NULL)
+	/* In case of non-GroupWise preferences edit, "prd" will be NULL. */
+	if (!prd || !prd->priv || !prd->priv->proxy_list)
 		return;
-	
+
 	priv = prd->priv;
-	for (;l; l = g_list_next (l)) {
-		aclInstance = (proxyHandler *) l->data;
+	list_iter = priv->proxy_list;
+	for (;list_iter; list_iter = g_list_next (list_iter)) {
+		aclInstance = (proxyHandler *) list_iter->data;
 
 		/* Handle case where the structure is new and deleted*/
 		if ( !((aclInstance->flags & E_GW_PROXY_NEW) && (aclInstance->flags & E_GW_PROXY_DELETED))) {
@@ -541,7 +566,7 @@ proxy_commit (GtkWidget *button, EConfigHookItemFactoryData *data)
 			if (aclInstance->flags & E_GW_PROXY_NEW )         	
 				e_gw_connection_add_proxy (prd->cnc, aclInstance);		
 
-			if (aclInstance->flags & E_GW_PROXY_DELETED)
+			if ((aclInstance->flags & E_GW_PROXY_DELETED) && !(aclInstance->flags & E_GW_PROXY_NEW))
 				e_gw_connection_remove_proxy (prd->cnc, aclInstance);
 
 			if (aclInstance->flags & E_GW_PROXY_EDITED)
@@ -549,11 +574,6 @@ proxy_commit (GtkWidget *button, EConfigHookItemFactoryData *data)
 		}
 	}
 
-	proxy_list = (GList *) g_object_get_data ((GObject *) account, "proxy_dialog");
-	g_list_foreach (proxy_list, (GFunc) free_proxy_handler, NULL);
-	g_list_free (proxy_list);
-	proxy_list = NULL;
-
 	g_object_unref (prd);
 	prd = NULL;
 }
@@ -590,7 +610,7 @@ proxy_update_tree_view (EAccount *account)
 	proxyDialog *prd = NULL;
     	GtkTreeIter iter;
 	GdkPixbuf *broken_image = NULL;
-	GList *l;
+	GList *list_iter;
 	proxyHandler *aclInstance;
 	gchar *file_name = e_icon_factory_get_icon_filename ("stock_person", E_ICON_SIZE_DIALOG);
 	proxyDialogPrivate *priv;
@@ -600,10 +620,10 @@ proxy_update_tree_view (EAccount *account)
 	broken_image = gdk_pixbuf_new_from_file (file_name, NULL);
 	
 	gtk_tree_store_clear (priv->store);
-	l = g_object_get_data ( (GObject *)account, "proxy_list") ;
+	list_iter = priv->proxy_list;
 
-	for (;l; l = g_list_next(l)) {        
-	        aclInstance = (proxyHandler *) l->data;
+	for (;list_iter; list_iter = g_list_next(list_iter)) {        
+	        aclInstance = (proxyHandler *) list_iter->data;
 
 		if(! (aclInstance->flags & E_GW_PROXY_DELETED )) {
 			gtk_tree_store_append (priv->store, &iter, NULL);
@@ -620,10 +640,10 @@ org_gnome_proxy (EPlugin *epl, EConfigHookItemFactoryData *data)
 {
 	EMConfigTargetAccount *target_account;
 	EAccount *account;
-	GList *proxy_list = NULL;
 	GtkButton *addProxy, *removeProxy, *editProxy;
 	proxyDialog *prd;
 	proxyDialogPrivate *priv;
+	int pag_num;
 
 	target_account = (EMConfigTargetAccount *)data->config->target;
 	account = target_account->account;
@@ -653,11 +673,7 @@ org_gnome_proxy (EPlugin *epl, EConfigHookItemFactoryData *data)
 			
 			prd->cnc = proxy_get_cnc(account);
 			
-			if (e_gw_connection_get_proxy_access_list(prd->cnc, &proxy_list)!= E_GW_CONNECTION_STATUS_OK) 
-				return NULL;
-			
-			g_object_set_data_full ((GObject *) account, "proxy_list", proxy_list, (GDestroyNotify) free_proxy_list);
-			proxy_update_tree_view (account);
+			priv->proxy_list = NULL;
 		} else {
 			GtkWidget *label;
 			priv->tab_dialog = gtk_vbox_new (TRUE, 10);
@@ -666,13 +682,16 @@ org_gnome_proxy (EPlugin *epl, EConfigHookItemFactoryData *data)
 		}	
 			
 		gtk_notebook_append_page ((GtkNotebook *)(data->parent), (GtkWidget *)priv->tab_dialog, gtk_label_new("Proxy"));
+		g_signal_connect ((GtkNotebook *)(data->parent), "switch-page", G_CALLBACK (proxy_page_changed_cb), account);
+		pag_num = gtk_notebook_page_num ((GtkNotebook *)(data->parent), (GtkWidget *)priv->tab_dialog);
+		g_object_set_data ((GObject *) account, "proxy_tab_num", pag_num);
 		gtk_widget_show_all (priv->tab_dialog);
 	}  else if (!g_strrstr (e_account_get_string(account, E_ACCOUNT_SOURCE_URL), "groupwise://")) {
 		prd = g_object_get_data ((GObject *) account, "prd");
 
 		if (prd) {
 			priv = prd->priv;
-			int pag_num;
+
 			if (priv) {
 			pag_num = gtk_notebook_page_num ( (GtkNotebook *)(data->parent), (GtkWidget *) priv->tab_dialog);
 			gtk_notebook_remove_page ( (GtkNotebook *)(data->parent), pag_num); 
@@ -682,6 +701,26 @@ org_gnome_proxy (EPlugin *epl, EConfigHookItemFactoryData *data)
 	return NULL;
 }
 
+static int
+proxy_page_changed_cb (GtkNotebook *notebook, GtkNotebookPage *page, int num, EAccount *account)
+{
+	proxyDialog *prd;
+	proxyDialogPrivate *priv;
+
+	if (!prd || !prd->priv || !prd->priv->proxy_list)
+		return;
+	
+	prd = g_object_get_data ((GObject *) account, "prd");
+	priv = prd->priv;
+
+	if (num == g_object_get_data ((GObject *) account, "proxy_tab_num") && account->enabled) {
+		if (e_gw_connection_get_proxy_access_list(prd->cnc, &priv->proxy_list)!= E_GW_CONNECTION_STATUS_OK) 
+				return NULL;
+		proxy_update_tree_view (account);	
+	}
+	return TRUE;
+}
+
 static void
 proxy_cancel(GtkWidget *button, EAccount *account)
 {
@@ -732,11 +771,17 @@ proxy_edit_ok (GtkWidget *button, EAccount *account)
 static proxyHandler * 
 proxy_get_item_from_list (EAccount *account, char *account_name)
 {
-	GList *l = g_object_get_data ((GObject *)account, "proxy_list");
+	proxyDialogPrivate *priv;
+	proxyDialog *prd = NULL;
+	GList *list_iter;
 	proxyHandler *iter;
+	
+	prd = g_object_get_data ((GObject *)account, "prd");    
+	priv = prd->priv;
+	list_iter = priv->proxy_list;
 
-	for (;l; l = g_list_next(l)) {        
-	        iter = (proxyHandler *) l->data;
+	for (;list_iter; list_iter = g_list_next(list_iter)) {        
+	        iter = (proxyHandler *) list_iter->data;
 
 		if ( g_str_equal (iter->proxy_email,account_name))
 			return iter;
