@@ -104,6 +104,9 @@ void emla_list_action_do (CamelFolder *folder, const char *uid, CamelMimeMessage
 	int send_message_response;
 	EAccount *account;
 
+	if (msg == NULL)
+		return;
+
 	for (t = 0; t < emla_n_action_headers; t++) {
 		if (emla_action_headers[t].action == action &&
 		    (header = camel_medium_get_header (CAMEL_MEDIUM (msg), emla_action_headers[t].header)) != NULL)
@@ -136,9 +139,7 @@ void emla_list_action_do (CamelFolder *folder, const char *uid, CamelMimeMessage
 		}
 		
 		/* get URL portion */
-		url = (char *) malloc (end - headerpos);
-		strncpy (url, headerpos, end - headerpos);
-		url[end-headerpos] = '\0';
+		url = g_strndup(headerpos, end - headerpos);
 
 		if (strncmp (url, "mailto:", 6) == 0) {
 			if (emla_action_headers[t].interactive)
@@ -165,7 +166,7 @@ void emla_list_action_do (CamelFolder *folder, const char *uid, CamelMimeMessage
 				goto exit;
 			g_error_free (err);			
 		}
-		free (url);
+		g_free (url);
 		url = NULL;
 		headerpos = end++;
 		
@@ -182,8 +183,7 @@ void emla_list_action_do (CamelFolder *folder, const char *uid, CamelMimeMessage
 exit:
 	free (action_data->uri);
 	free (action_data);
-	if (url)
-		free(url);
+	g_free(url);
 }
 
 void emla_list_help (EPlugin *item, EMMenuTargetSelect* sel)
